package oapi;

import olif.Olif;
import olif.Entry;
import olif.EntryType;
import olif.Transfer;
import olif.KeyDCType;

import required.Required;
import required.GlobalType;
import required.LocalType;
import required.GlobalTypeItem;
import required.LocalTypeItem;

import support.csv.CSV;

import java.io.File;
import java.io.FileInputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.FileReader;
import java.io.FileWriter;

import java.util.StringTokenizer;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Vector;
import java.util.List;

import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import org.apache.xpath.XPathAPI;
import org.apache.xerces.dom.DOMImplementationImpl;

import javax.xml.parsers.DocumentBuilder; 
import javax.xml.parsers.DocumentBuilderFactory;  
import javax.xml.parsers.FactoryConfigurationError;  
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.stream.StreamSource;

/**
* The OAPIConvert class provides static functions and methods to convert OLIF Documents to and from CSV and arbitrary XML files.
* A typical invokation looks like this:
* <p><code>
* import olif;<br>
* import oapi;<br>
* File configFile = new File("config.xml");<br>
* File inputFile = new File("input.csv");<br>
* File outputFile = new File("output.xml");<br>
* <br>
* Olif template = OAPI.convertCSVToOLIF(configFile, inputFile, outputFile);
* </code></p><p>
* which creates an OLIF file <code>output.xml</code> which is equivalent to the CSV file.
* The rules for the transformation are defined in <code>config.xml</code>.
* </p>
*/
public class OAPIConvert {

/**
* <p> This converts a CSV file to an OLIF document. The config file fills the required elements. </p>
*
* @param configFile XML file with information about obligatory elements
* @param inputFile CSV file with specific format (int id, String sourceCanForm, String targetCanForm)
* @param outputFile XML file to write OLIF object to
*
* @throws OAPIConvertException 
*/
	public static void convertCSVToOlif(File configFile, File inputFile, File outputFile) throws OAPIConvertException {
		CSV csv;
		Olif olif;
		int count;
		BufferedReader bReader;
		Entry entrySource, entryTarget;
		String line, id, source, target;
		Required required;
		Transfer refTransfer;

		count = 0;
		try {
			csv = new CSV(new FileReader(inputFile));
			required = OAPISerialize.readConfig(configFile);
			olif = OAPICreate.createOlifTemplate(required);
			System.out.println("parsing: "+ inputFile + "...");
			Enumeration e = csv.parse();
			System.out.println("parsing finished.\n");
			while (e.hasMoreElements()) {
				id = (String)e.nextElement();
				source = (String)e.nextElement();
				target = (String)e.nextElement();
				System.out.println("DEBUG:" + id + ":" + source + ":" + target);
				System.out.println("converting: "+ source);
				entrySource = OAPICreate.createEntryTemplate(required, new Integer(id).toString());
				(((KeyDCType)((entrySource.getMono()).getKeyDC())).getCanForm()).setContent(source);
				entryTarget = OAPICreate.createEntryTemplate(required, new Integer(id).toString());
				(((KeyDCType)((entryTarget.getMono()).getKeyDC())).getCanForm()).setContent(target);
				String refUUID = entryTarget.getConceptUniversalId();
				refTransfer = new Transfer();
				refTransfer.setTrTarget(refUUID);
				entrySource.addTransfer(refTransfer);
				OAPIModify.addEntryToOlif(olif, entrySource);
				System.out.println("...establish target link to: "+ target);
				OAPIModify.addEntryToOlif(olif, entryTarget);
			}
			OAPISerialize.writeOlif(olif, outputFile);
		} catch (Exception e) {
			throw new OAPIConvertException("Error occured during CSV conversion: "+e.getMessage());
		}
	}

/**
* <p> This converts a CSV file to an OLIF document. The config file fills the required elements, which is taken from a former created OAPITemplate. </p>
*
* @param oapiT OAPITemplate with encapsulated Required object (faster)
* @param inputFile CSV file with specific format (int id, String sourceCanForm, String targetCanForm)
* @param outputFile XML file to write OLIF object to
*
* @throws OAPIConvertException 
*/
	public static void convertCSVToOlif(OAPITemplate oapiT, File inputFile, File outputFile) throws OAPIConvertException {
		CSV csv;
		Olif olif;
		int count;
		BufferedReader bReader;
		Entry entrySource, entryTarget;
		String line, id, source, target;
		Required required;
		Transfer refTransfer;

		count = 0;
		try {
			csv = new CSV(new FileReader(inputFile));
			required = oapiT.getRequired();
			olif = oapiT.getOlifTemplate();
			System.out.println("parsing: "+ inputFile + "...");
			Enumeration e = csv.parse();
			System.out.println("parsing finished.\n");
			while (e.hasMoreElements()) {
				id = (String)e.nextElement();
				source = (String)e.nextElement();
				target = (String)e.nextElement();
				System.out.println("DEBUG:" + id + ":" + source + ":" + target);
				System.out.println("converting: "+ source);
				entrySource = oapiT.getEntryTemplate(id);
				(((KeyDCType)((entrySource.getMono()).getKeyDC())).getCanForm()).setContent(source);
				entryTarget = oapiT.getEntryTemplate(id);
				(((KeyDCType)((entryTarget.getMono()).getKeyDC())).getCanForm()).setContent(target);
				String refUUID = entryTarget.getConceptUniversalId();
				refTransfer = new Transfer();
				refTransfer.setTrTarget(refUUID);
				entrySource.addTransfer(refTransfer);
				OAPIModify.addEntryToOlif(olif, entrySource);
				System.out.println("...establish target link to: "+ target);
				OAPIModify.addEntryToOlif(olif, entryTarget);
			}
			OAPISerialize.writeOlif(olif, outputFile);
		} catch (Exception e) {
			e.printStackTrace();
			throw new OAPIConvertException("Error occured during CSV conversion: "+e.getMessage()+"\n"+e.toString());
		}
	}

/**
* <p> This converts an arbitrary XML file to an OLIF document. The config file fills the required elements and the transform file specifies the transformation rules. </p>
*
* @param configFile XML file with information about obligatory elements
* @param transformFile XSL file with transformation rules. In case of conflicts, the config file has higher priority
* @param inputFile XML file with arbitrary format 
* @param outputFile XML file to write OLIF object to
*
* @throws OAPIConvertException 
*/
	public static void convertXMLToOlif(File configFile, File transformFile, File inputFile, File outputFile) throws OAPIConvertException {
       	Olif olif;
		Required required;
       	Document n;
		try {
		
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        	DocumentBuilder builder = factory.newDocumentBuilder();
        	Document d = builder.newDocument();  
			DOMResult dr = new DOMResult(d);
			TransformerFactory tFactory = TransformerFactory.newInstance();
			Transformer transformer = tFactory.newTransformer(new StreamSource(transformFile));
			transformer.transform(new StreamSource(inputFile), dr);
			required =  OAPISerialize.readConfig(configFile);
			Node o = dr.getNode();
			olif = OAPICreate.createOlifTemplate(required);
		
			Element secondRoot = ((Document)o).getDocumentElement();
			NodeList entryList = secondRoot.getElementsByTagName("entry");
    	    for (int i = 0; i < entryList.getLength(); i++) {
				n = OAPICreate.createNewEntryDocument();
				NodeList contentList = ((Element)entryList.item(i)).getChildNodes();
	    	    for (int j = 0; j < contentList.getLength(); j++) {
		            Element firstRoot = n.getDocumentElement();
	            	Element oneToMove = (Element)contentList.item(j);
	        	    Node newOneToMove = n.importNode(oneToMove, true);
    		        firstRoot.appendChild(newOneToMove);
					}
				Entry e = OAPIModify.enhanceEntryWithTemplate(n, required, "1");
				System.out.println("converting: "+ e.getMono().getKeyDC().getCanForm() );
				OAPIModify.addEntryToOlif(olif, e);
			}
			OAPISerialize.writeOlif(olif, outputFile);
		} catch (Exception e) {
			throw new OAPIConvertException("Error occured while converting XML: " + e.getMessage());
		}
	}
/**
* <p> This converts an arbitrary XML file to an OLIF document. The config file fills the required elements, which are taken from a former created OAPITemplate object, and the transform file specifies the transformation rules. </p>
*
* @param oapiT OAPITemplate object with encapsulated Required object
* @param transformFile XSL file with transformation rules. In case of conflicts, the config file has higher priority
* @param inputFile XML file with arbitrary format 
* @param outputFile XML file to write OLIF object to
*
* @throws OAPIConvertException 
*/
	public static void convertXMLToOlif(OAPITemplate oapiT, File transformFile, File inputFile, File outputFile) throws OAPIConvertException {
       	Olif olif;
       	Document n;
		try {
		
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        	DocumentBuilder builder = factory.newDocumentBuilder();
        	Document d = builder.newDocument();  
			DOMResult dr = new DOMResult(d);
			TransformerFactory tFactory = TransformerFactory.newInstance();
			Transformer transformer = tFactory.newTransformer(new StreamSource(transformFile));
			transformer.transform(new StreamSource(inputFile), dr);
			Node o = dr.getNode();
			olif = oapiT.getOlifTemplate();
		
			Element secondRoot = ((Document)o).getDocumentElement();
			NodeList entryList = secondRoot.getElementsByTagName("entry");
    	    for (int i = 0; i < entryList.getLength(); i++) {
				n = OAPICreate.createNewEntryDocument();
				NodeList contentList = ((Element)entryList.item(i)).getChildNodes();
	    	    for (int j = 0; j < contentList.getLength(); j++) {
		            Element firstRoot = n.getDocumentElement();
	            	Element oneToMove = (Element)contentList.item(j);
	        	    Node newOneToMove = n.importNode(oneToMove, true);
    		        firstRoot.appendChild(newOneToMove);
				}
				Entry e = OAPIModify.enhanceEntryWithTemplate(n, oapiT.getRequired(), "1");
				System.out.println("converting: "+ e.getMono().getKeyDC().getCanForm() );
				OAPIModify.addEntryToOlif(olif, e);
			}
			OAPISerialize.writeOlif(olif, outputFile);
		} catch (Exception e) {
			throw new OAPIConvertException("Error occured while converting XML: " + e.getMessage());
		}
	}
	
/**
* <p> This converts an arbitrary XML file to an OLIF document. The configuration consists of the required elements, which are taken from a former created OAPITemplate object, and the transformed XML document, which is taken from a former created OAPIXSLTemplate object. </p>
*
* @param oapiT OAPITemplate object with encapsulated Required object
* @param transT OAPIXSLTemplate object with encapsulated transformed XML document object
* @param inputFile XML file with arbitrary format 
* @param outputFile XML file to write OLIF object to
*
* @throws OAPIConvertException 
*/
	public static void convertXMLToOlif(OAPITemplate oapiT, OAPIXSLTemplate transT, File inputFile, File outputFile) throws OAPIConvertException {
		Olif olif;
		Document n;
		try {
			Node o = transT.getXSLTemplateNode();
			olif = oapiT.getOlifTemplate();
		
			Element secondRoot = ((Document)o).getDocumentElement();
			NodeList entryList = secondRoot.getElementsByTagName("entry");
    	    for (int i = 0; i < entryList.getLength(); i++) {
				n = OAPICreate.createNewEntryDocument();
				NodeList contentList = ((Element)entryList.item(i)).getChildNodes();
	    	    for (int j = 0; j < contentList.getLength(); j++) {
		            Element firstRoot = n.getDocumentElement();
	            	Element oneToMove = (Element)contentList.item(j);
	        	    Node newOneToMove = n.importNode(oneToMove, true);
    		        firstRoot.appendChild(newOneToMove);
				}
				Entry e = OAPIModify.enhanceEntryWithTemplate(n, oapiT.getRequired(), "1");
				System.out.println("converting: "+ e.getMono().getKeyDC().getCanForm() );
				OAPIModify.addEntryToOlif(olif, e);
			}
			OAPISerialize.writeOlif(olif, outputFile);
		} catch (Exception e) {
			throw new OAPIConvertException("Error occured while converting XML: " + e.getMessage());
		}
	}

}