package oapi;

import java.io.File;
import java.io.FileReader;

import java.util.Enumeration;
import java.util.Iterator;

import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import org.apache.xerces.dom.DOMImplementationImpl;

import required.Required;
import required.GlobalType;
import required.LocalType;
import required.GlobalTypeItem;
import required.LocalTypeItem;

import olif.Olif;
import olif.Entry;
import olif.EntryType;
import olif.Transfer;
import olif.KeyDCType;

import support.uuid.UUID;

/*
 * import castor (un-)marshalling 
 * and exception classes
 */
import org.exolab.castor.xml.Marshaller;
import org.exolab.castor.xml.Unmarshaller;
import org.exolab.castor.xml.CastorException;

/**
* The OAPICreate class provides static functions and methods to create OLIF Documents from scratch.
* A typical invokation looks like this:
* <p><code>
* import olif;<br>
* import oapi;<br>
* Olif template = OAPI.createOlifTemplate(Required required);
* </code></p><p>
* which creates an OLIF template object with the required elements included.
* </p>
*/
public class OAPICreate {


/**
* <p> This creates an OLIF object from an OLIF XML file. </p>
*
* @param olifXMLfile OLIF file to create OLIF object from
* @return OLIF object
* @throws OAPICreateException
*/
	public static Olif createOlifObject(File olifXMLFile) throws OAPICreateException {
		try {
			FileReader reader = null;
			reader = new FileReader(olifXMLFile);
			return (Olif)Unmarshaller.unmarshal(Olif.class, reader);
		} catch (Exception e) {
			throw new OAPICreateException("Error occured while	unmarshalling document: " + e.getMessage());
		}
	}

/**
* <p> This creates an OLIF Document object from an OLIF XML File. </p>
*
* @param olifXMLfile Olif file to create OLIF document object from
* @return OLIF document object
* @throws OAPICreateException
*/
	public static Document createOlifDocument(File olifXMLFile) throws OAPICreateException {
		try {
			return OAPISerialize.convertToDocument(OAPICreate.createOlifObject(olifXMLFile));
		} catch (Exception e) {
			throw new OAPICreateException("Error occured while converting document: " + e.getMessage());
		}
	}

/**
* <p> This creates a new OLIF document object from scratch. </p>
* 
* @return OLIF document object 
*/
	public static Document createNewOlifDocument() {
		DOMImplementation domImpl = new DOMImplementationImpl();
		Document olifDocument = domImpl.createDocument(null, "olif", null);
		// OLIF entry has to include a body
		Element body = olifDocument.createElement("body");
		// append body to OLIF entry
		Element root = olifDocument.getDocumentElement();
		root.appendChild(body);
		return olifDocument;
	} 	

/**
* <p> This creates a new Entry node from scratch. </p>
* 
* @return Entry document object
*/
	public static Document createNewEntryDocument() {
		DOMImplementation domImpl = new DOMImplementationImpl();
		return domImpl.createDocument(null, "entry", null);
	} 

/**
* <p> This creates a new OLIF object with obligatory elements already filled in. </p>
*
* @param required required elements taken from a config file
* @return OLIF object
* @throws OAPICreateException, OAPISerializeException, OAPIException
*/
	public static Olif createOlifTemplate(Required required) throws OAPICreateException, OAPISerializeException, OAPIException {
		Document olifDocument;
	
		olifDocument = createNewOlifDocument();
		Enumeration e = ((GlobalType)(required.getGlobal())).enumerateGlobalTypeItem();

		while(e.hasMoreElements()) {
			GlobalTypeItem gti = (GlobalTypeItem)(e.nextElement());
			olifDocument = OAPIModify.insertElement(olifDocument, 
										(gti.getDefault()).getType().toString(),
										(gti.getDefault()).getPath(), 
										(gti.getDefault()).getValue());
		}
		return OAPISerialize.convertToOlif(olifDocument);
	}

/**
* <p> This creates a new Entry object with obligatory elements already filled in. </p>
*
* @param required required elements taken from a config file
* @param id locally unique String to identify Entry
* @return Entry object
* @throws OAPICreateException, OAPISerializeException, OAPIException
*/
	public static Entry createEntryTemplate(Required required, String id) throws OAPICreateException, OAPISerializeException, OAPIException {
		Node entryNode;
		Entry entryObject;	
	
		entryNode = createNewEntryDocument();
		Enumeration e = ((LocalType)(required.getLocal())).enumerateLocalTypeItem();

		while(e.hasMoreElements()) {
			LocalTypeItem lti = (LocalTypeItem)(e.nextElement());
			entryNode = OAPIModify.insertElementIntoEntry(entryNode,
												(lti.getDefault()).getType().toString(), 
												(lti).getDefault().getPath(), 
												(lti).getDefault().getValue());
		} 
		entryObject = OAPISerialize.convertToEntry((Document)entryNode);
		entryObject.setConceptUniversalId(getUUID()); 
		entryObject.setConceptUserId(id);
		return entryObject;
	}

/**
* <p> This creates a UUID for each Entry node.</p>
* @return UUID
*/
	private static String getUUID() {
		UUID uuid = new UUID();
		return uuid.toString();
	}

}