package oapi;

import java.io.File;
import java.io.FileInputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.StringTokenizer;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Vector;
import java.util.List;

import support.uuid.UUID;

import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import support.csv.CSV;
import support.spellcheck.SpellerBean;
import support.spellcheck.SpellDictionary;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.xpath.XPathAPI;
import org.apache.xerces.dom.DOMImplementationImpl;

import required.Required;
import required.GlobalType;
import required.LocalType;
import required.GlobalTypeItem;
import required.LocalTypeItem;

import olif.Olif;
import olif.Entry;
import olif.EntryType;
import olif.Transfer;
import olif.KeyDCType;

/**
* The OAPIModify class provides static functions and methods to modify OLIF documents, for example change Entries, perform spellcheck, etc.
* A typical invokation looks like this:
* <p><code>
* import olif;<br>
* import oapi;<br>
* Entry oldEntry, newEntry;<br>
* <br>
* newEntry = OAPICreate.createEntryTemplate();<br>
* Enumeration e = (olif.getBody()).enumerateEntry();<br>
* oldEntry = (EntryType)e.nextElement();<br>
* Olif newOlif = exchangeOlifEntry(OlifObject, oldEntry, newEntry);
* </code>
* </p><p>
* which exchanges an OLIF Entry and saves the new OLIF object in <code>newOlif</code>.
* </p>
*/

public class OAPIModify {

/**
* <p> This spellchecks an Entry object on the <code>canForm</code> element with
the dictionary <code>dict</code>. </p>
*
* @param entry Entry object to be spellchecked
* @param dict dictionary file
* @return List with corrections
*/
	public static List spellcheck(Entry entry, File dict) {
		SpellerBean sbean = new SpellerBean();
		SpellDictionary dictionary = new SpellDictionary();
		dictionary.createDictionary(dict);
		String word = (((KeyDCType)((entry.getMono()).getKeyDC())).getCanForm()).getContent();
		sbean.setWord(word);
		sbean.setCode(dictionary.getCode(word));
		if (!(dictionary.isCorrect(word))) {
			return dictionary.getSuggestions(word);
		} else {
			return null;
		}
	}

/**
* <p> This inserts an element into an OLIF object. </p>
*
* @param olif OLIF object to insert the Entry object into
* @param entry Entry object to be added to the OLIF object
* @return modified OLIF object
*/
	public static Olif addEntryToOlif(Olif olif, Entry entry) {
		(olif.getBody()).addEntry(entry);
		return olif;
	}

/**
* <p> This exchanges two Entries in an Olif object. </p>
*
* @param olif OLIF object to operate on
* @param entryOld Entry object to be exchanged
* @param entryNew new Entry object
* @return modified OLIF object
*/
	public static Olif exchangeOlifEntry(Olif olif, Entry entryOld, Entry entryNew) {
		Iterator i;
		List newEntryList;
		Entry entryIter;
	
		i = getEntryIterator(olif);	
		newEntryList = new Vector();
		while (i.hasNext()) {
			entryIter = (Entry)i.next();
			if ((entryIter).getConceptUniversalId().equals(entryOld.getConceptUniversalId() )) {
				newEntryList.add(entryNew);
			} else {
				newEntryList.add(entryIter);
			}
		}
		(olif.getBody()).removeAllEntry();
		i = newEntryList.iterator();
		while (i.hasNext()) {
			olif = addEntryToOlif(olif, (Entry)i.next());
		}
		return olif;
	}

/**
* <p> This returns an iterator for all Entry objects in an OLIF object.</p>
*
* @param olif OLIF object to get Entry objects from
* @return Iterator
*/
	public static Iterator getEntryIterator(Olif olif) {
		EntryType entryIter;
		List newEntryList;
		
		newEntryList = new Vector();
		Enumeration e = (olif.getBody()).enumerateEntry();
		while (e.hasMoreElements()) {
			entryIter = (EntryType)e.nextElement();
			newEntryList.add(entryIter);
		}
		return newEntryList.iterator();
	}	

/**
* <p> This enhances an Entry object according to the simplified XPath target and the value. </p>
*
* @param entry Entry object to be enhanced
* @param type String with simplified XPath structure
* @param value String with value for element
* @return modified Entry object
*/
	public static Entry enhanceEntry(Entry entry, String type, String path, String value) throws OAPIException {
		try {
			return OAPISerialize.convertToEntry((Document)(insertElementIntoEntry(OAPISerialize.convertToDocument(entry), type, path, value)));
		} catch (Exception e) {
			throw new OAPIException("Error occured while enhancing entry: " + e.getMessage());
		}
	}

/**
* <p> This inserts elements at <code>path</code> with value <code>value</code> into OLIF document objects. Used by <code>createOlifTemplate</code>. </p>
*
* @param olifDocument OLIF document object to insert elements to
* @param type type of element to be inserted
* @param path simplified XPATH expression which points to the desired location in the document
* @param value value for the test node
* @return modified OLIF document object
* @throws OAPIException if root element is not \<olif\> 
*/
	public static Document insertElement(Document olifDocument, String type, String path, String value) throws OAPIException { 
		String xpath;
		StringTokenizer st;
		Element root;
		NodeList nl = null;
		Node nodeIter;
		String next = null;

		try {
			st = new StringTokenizer(path, "/");
			xpath = "/" + st.nextElement();
			if (!xpath.equals("/olif")) {
				throw new Exception("root element must be <olif>");
			}
			root = olifDocument.getDocumentElement();
			nodeIter = root;
			while (st.hasMoreElements()) {
				next = (String)st.nextElement();
				nl = XPathAPI.selectNodeList(olifDocument, xpath + "/" + next);
				if (nl.getLength()==0) {
					nodeIter = XPathAPI.selectSingleNode(olifDocument, xpath); 
					if (type.equals("att") && (!(st.hasMoreElements()))) {
						if (((Element)nodeIter).hasAttribute(next)) {
							((Element)nodeIter).removeAttribute(next);
						}
						Attr lastAttr = (olifDocument).createAttribute(next);
						lastAttr.setValue(value);
						((Element)nodeIter).setAttributeNode(lastAttr);
					} else {
						Element lastElement = olifDocument.createElement(next);
						nodeIter = addChild(nodeIter, lastElement);
						String innerNext = null;
						while (st.hasMoreElements()) {
							innerNext = (String)st.nextElement();
							lastElement = olifDocument.createElement(innerNext);
							nodeIter = addChild(nodeIter, lastElement);
						}
						if (type.equals("cn")) {
							lastElement.appendChild(olifDocument.createTextNode(value));
						}
					}
				} else {
						nodeIter = XPathAPI.selectSingleNode(olifDocument, xpath);
						xpath = xpath + "/" + next;
				}
			}
			if (nl.getLength()>0) {
				Element appendElement = olifDocument.createElement(next);
				if (type.equals("cn")) {
					appendElement.appendChild(olifDocument.createTextNode(value));
				}
				nodeIter = addChild(nodeIter, appendElement);
			}
			return olifDocument;
		} catch (Exception e) {
			throw new OAPIException("Error occured during inserting element: " + e.getMessage());
		}			
	}

/**
* <p> This inserts elements at "path" with value "value" into an Entry node. Used by <code>createEntryTemplate</code>. </p>
*
* @param entryNode Entry node to insert elements to
* @param type type of element to be inserted
* @param path simplified XPATH expression which points to the desired location 
* @param value value for the test node. "null" value creates no text node
* @return modified Entry node object
* @throws OAPIException
*/
	public static Node insertElementIntoEntry(Node entryNode, String type, String path, String value) throws OAPIException {
		StringTokenizer st;
		String xpath;
		Element root;
		NodeList nl = null;
		Node nodeIter;
		String next = null;

		try {
			st = new StringTokenizer(path, "/");
			xpath = "/" + st.nextElement();
			if (!xpath.equals("/entry")) {
				throw new Exception("root element must be <entry>");
			}
			root = ((Document)entryNode).getDocumentElement();
			nodeIter = root;
			while (st.hasMoreElements()) {
				next = (String)st.nextElement();
				nl = XPathAPI.selectNodeList(entryNode, xpath + "/" + next);
				if (nl.getLength()==0) {
					nodeIter = XPathAPI.selectSingleNode(entryNode, xpath); 
					if (type.equals("att") && (!(st.hasMoreElements()))) {
						Attr lastAttr = ((Document)(entryNode)).createAttribute(next);
						lastAttr.setValue(value);
						((Element)nodeIter).setAttributeNode(lastAttr);
					} else {
						Element lastElement = ((Document)(entryNode)).createElement(next);
						nodeIter = addChild(nodeIter, lastElement);
						String innerNext = null;
						while (st.hasMoreElements()) {
							innerNext = (String)st.nextElement();
							lastElement = ((Document)(entryNode)).createElement(innerNext);
							nodeIter = addChild(nodeIter, lastElement);
						}
						if (type.equals("cn")) {
							lastElement.appendChild(((Document)(entryNode)).createTextNode(value));
						}
					}
				} else {
					nodeIter = XPathAPI.selectSingleNode(entryNode, xpath);
					xpath = xpath + "/" + next;
				}
			}
			if (nl.getLength()>0) {
				// for future release: get Castor properties - element unique?
				Element appendElement = ((Document)(entryNode)).createElement(next);
				if (type.equals("cn")) {
					appendElement.appendChild(((Document)(entryNode)).createTextNode(value));
				}
				nodeIter = addChild(nodeIter, appendElement);
			}
			return entryNode;
		} catch (Exception e) {
			throw new OAPIException("Error occured during inserting element: " + e.getMessage());
		}			
	}

/**
* <p> This inserts an Element at "path" with value "value" into an Entry node if the Element does not exist. Used by <code>createEntryTemplate</code>. </p>
*
* @param entryNode Entry node to insert elements to
* @param path simplified XPATH expression which points to the desired location 
* @param value value for the test node. "null" value creates no text node
*
* @return new Node with inserted Element, if it did not exist before
* @throws OAPIConvertException
*/
	public static Node insertElementIntoEntryIfNotExists(Node entryNode, String type, String path, String value) throws OAPIConvertException {
		StringTokenizer st;
		String xpath;
		Element root;
		NodeList nl = null;
		Node nodeIter;
		String next = null;

		try {
			st = new StringTokenizer(path, "/");
			xpath = "/" + st.nextElement();
			if (!xpath.equals("/entry")) {
				throw new Exception("root element must be <entry>");
			}
			root = ((Document)entryNode).getDocumentElement();
			nodeIter = root;
			while (st.hasMoreElements()) {
				next = (String)st.nextElement();
				nl = XPathAPI.selectNodeList(entryNode, xpath + "/" + next);
				if (nl.getLength()==0) {
					nodeIter = XPathAPI.selectSingleNode(entryNode, xpath); 
					if (type.equals("att") && (!(st.hasMoreElements()))) {
						Attr lastAttr = ((Document)(entryNode)).createAttribute(next);
						lastAttr.setValue(value);
						((Element)nodeIter).setAttributeNode(lastAttr);
					} else {
					    Element lastElement = ((Document)(entryNode)).createElement(next);
						nodeIter = addChild(nodeIter, lastElement);
						String innerNext = null;
						while (st.hasMoreElements()) {
							innerNext = (String)st.nextElement();
							lastElement = ((Document)(entryNode)).createElement(innerNext);
							nodeIter = addChild(nodeIter, lastElement);
						}
						if (type.equals("cn")) {
							lastElement.appendChild(((Document)(entryNode)).createTextNode(value));
						}
					}
				} else {
					nodeIter = XPathAPI.selectSingleNode(entryNode, xpath);
					xpath = xpath + "/" + next;
				}
			}
			return entryNode;
		} catch (Exception e) {
			throw new OAPIConvertException("Error occured during inserting element: " + e.getMessage());
		}			
	}
	
/**
* <p> This creates a new Entry object with obligatory elements already filled in (if they are not already present). </p>
*
* @param entryNode Entry node object to be enhanced
* @param required required elements taken from a config file
* @param id locally unique String to identify Entry
* @return Entry object
* @throws OAPICreateException, OAPISerializeException, OAPIException
*/

	public static Entry enhanceEntryWithTemplate(Node entryNode, Required required, String id) throws OAPIException {
		Entry entryObject;	
	
		Enumeration e = ((LocalType)(required.getLocal())).enumerateLocalTypeItem();

		while(e.hasMoreElements()) {
			LocalTypeItem lti = (LocalTypeItem)(e.nextElement());
			entryNode = OAPIModify.insertElementIntoEntryIfNotExists(entryNode,
												(lti.getDefault()).getType().toString(), 
												(lti).getDefault().getPath(), 
												(lti).getDefault().getValue());
		} 
		entryObject = OAPISerialize.convertToEntry((Document)entryNode);
		entryObject.setConceptUniversalId(getUUID()); 
		entryObject.setConceptUserId(id);
		return entryObject;
	}

/**
* <p> This add a child element node to the parent node. </p>
*
* @param parent node to append element to
* @param element element which is to append
*
* @return modified Node
*/
	private static Node addChild(Node parent, Element element) {
		parent.appendChild(element);
		return element;
	}


/**
* <p> This creates a UUID for each Entry node.</p>
* @return UUID
*/
	private static String getUUID() {
		UUID uuid = new UUID();
		return uuid.toString();
	}

}
