package oapi;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import org.w3c.dom.Document;

/*
 * import castor (un-)marshalling 
 * and exception classes
 */
import org.exolab.castor.xml.Marshaller;
import org.exolab.castor.xml.Unmarshaller;
import org.exolab.castor.xml.CastorException;

/*
 * import castor generated classes
 */
import olif.Olif;
import olif.Entry;
import required.Required;


/**
* The OAPISerialize class provides static functions and methods to serialize OLIF objects and OLIF document objects to XML files and convert them respectively.
* A typical invokation looks like this:
* <p><code>
* import olif;<br>
* import oapi;<br>
* <br>
* Olif template = OAPICreate.createOlifTemplate(Required required);<br>
* Document olifDoc = OAPISerialize.convertToDocument(template);
* </code></p><p>
* which creates an OLIF object and converts it to a OLIF document object.
* </p>
*/
public class OAPISerialize {
	
/**
* <p> This converts an OLIF document object to an OLIF object. </p>
*
* @param olifDocument OLIF document object to convert to OLIF object
* @return new OLIF object
* @throws OAPISerializeException
*/
	public static Olif convertToOlif(Document olifDocument) throws OAPISerializeException {
		try {
			return (Olif)Unmarshaller.unmarshal(Olif.class, olifDocument);
		} catch (Exception e) {
			throw new OAPISerializeException("Error occured while unmarshalling document: " + e.getMessage());
		}
	} 

/**
* <p> This converts an OLIF object to an OLIF document object. </p>
*
* @param olifObject OLIF object to convert to OLIF document object
* @return new OLIF document object
* @throws OAPISerializeException
*/
	public static Document convertToDocument(Olif olifObject) throws OAPISerializeException {
		try {
			Document output = new org.apache.xerces.dom.DocumentImpl();			Marshaller ms = new Marshaller(output);
			ms.setNamespaceMapping("", "http://www.olif.net");
			ms.setSchemaLocation("http://www.olif.net schemas/olifxsd/olif.xsd");
			ms.marshal(olifObject);
			return output;
		} catch (Exception e) {
			throw new OAPISerializeException("Error occured while unmarshalling document: " + e.getMessage());
		}	
	} 

/**
* <p> This converts an converts an Entry document object to an Entry object. </p>
*
* @param entryDocument Entry document object to convert to Entry object
* @return new Entry object
* @throws OAPISerializeException
*/
	public static Entry convertToEntry(Document entryDocument) throws OAPISerializeException {
		try {
			return (Entry)Unmarshaller.unmarshal(Entry.class, entryDocument);
		} catch (Exception e) {
			throw new OAPISerializeException("Error occured while unmarshalling document: " + e.getMessage());
		}
	} 

/**
* <p> This converts an Entry object to an Entry document object. </p>
*
* @param entryObject Entry object to convert to Entry document
* @return new Entry document object
* @throws OAPISerializeException
*/
	public static Document convertToDocument(Entry entryObject) throws OAPISerializeException {
		try {
			Document output = new org.apache.xerces.dom.DocumentImpl();
			Marshaller.marshal(entryObject, output);
			return output;
		} catch (Exception e) {
			throw new OAPISerializeException("Error occured while unmarshalling document: " + e.getMessage());
		}
	} 


/**
* <p> This reads the config file where obligatory elements are listed and returns <code>Required</code> object. </p>
*
* @param configFile XML file with information about obligatory elements
* @return Required object
* @throws OAPISerializeException
*/
	public static Required readConfig(File configFile) throws OAPISerializeException {
		try {
			FileReader reader = new FileReader(configFile);
			return (Required)Unmarshaller.unmarshal(Required.class, reader);
		} catch (Exception e) {
			throw new OAPISerializeException("Error occured while reading config file: " + e.getMessage());
		}
	}

/**
* <p> This writes an OLIF XML file from an OLIF object. </p>
*
* @param olifObject OLIF object to write to file
* @param outputOlifXMLFile file to write OLIF object to
* @throws OAPISerializeException
*/
	public static void writeOlif(Olif olifObject, File outputOlifXMLFile) throws OAPISerializeException {
		try {
			FileWriter writer = new FileWriter(outputOlifXMLFile);
			Marshaller ms = new Marshaller(writer);
			ms.setNamespaceMapping("", "http://www.olif.net");
			ms.setSchemaLocation("http://www.olif.net schemas/olifxsd/olif.xsd");
			ms.marshal(olifObject);
		} catch (Exception e) {
			throw new OAPISerializeException("Error occured during writing Olif object: " + e.getMessage());
		}
	} 

/**
* <p> This writes an OLIF XML file from an OLIF document object. </p>
*
* @param olifDocument OLIF document to write to file
* @param outputOlifXMLFile file to write OLIF document to
* @throws OAPISerializeException {
*/
	public static void writeOlif(Document olifDocument, File outputOlifXMLFile) throws OAPISerializeException {
		try {
			writeOlif(convertToOlif(olifDocument), outputOlifXMLFile);
		} catch (Exception e) {
			throw new OAPISerializeException("Error occured during writing Olif document: " + e.getMessage());
		}
	}

}
