package oapi;

import java.io.File;
import org.w3c.dom.Document;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import java.util.Enumeration; 
import support.uuid.UUID;

import required.*;
import olif.*;

/**
* The OAPITemplate class provides dynamic functions and methods for repetitional tasks like conversions.
* A typical invokation looks like this:
* <p><code>
* import olif;<br>
* import oapi;<br>
* File configFile = new File("config.xml");<br>
* File inputFile = new File("input.csv");<br>
* File outputFile = new File("output.xml");<br>
* OAPITemplate oapiT = new OAPITemplate(configFile);<br>
* OAPIConvert.convertCSVToOlif(oapiT, inputFile, outputFile);
* </code></p><p>
* which uses the OAPITemplate to convert the CSV file and therefore reads the config file only once.
* </p>
*/
public class OAPITemplate {

	Node entryTemplateNode;
	Document olifTemplateDocument;
	Required required;
	
/**
* <p> This creates the OAPITemplate object and makes all arrangements. </p>
*
* @param configFile XML config file with required elements
* @throws OAPIException
*/
	public OAPITemplate(File configFile) throws OAPIException {
		required = OAPISerialize.readConfig(configFile);
		olifTemplateDocument = createOlifTemplateDocument(required);
		entryTemplateNode = createEntryTemplateNode(required);
	}

	
/**
* <p> This creates a new OLIF document object with obligatory elements already filled in. </p>
*
* @param required required elements taken from a config file
* @return OLIF document object
* @throws OAPIException
*/
	private Document createOlifTemplateDocument(Required required) throws OAPIException {
		Document olifDocument;
	
		olifDocument = OAPICreate.createNewOlifDocument();
		Enumeration e = ((GlobalType)(required.getGlobal())).enumerateGlobalTypeItem();

		while(e.hasMoreElements()) {
			GlobalTypeItem gti = (GlobalTypeItem)(e.nextElement());
			olifDocument = OAPIModify.insertElement(olifDocument, 
										(gti.getDefault()).getType().toString(),
										(gti.getDefault()).getPath(), 
										(gti.getDefault()).getValue());
		}
		return olifDocument;
	}

/**
* <p> This creates a new Entry document object with obligatory elements already filled in. </p>
*
* @param required required elements taken from a config file
* @return Entry node object
* @throws OAPIException
*/
	private Node createEntryTemplateNode(Required required) throws OAPIException {
		Node entryNode;
	
		entryNode = OAPICreate.createNewEntryDocument();
		Enumeration e = ((LocalType)(required.getLocal())).enumerateLocalTypeItem();

		while(e.hasMoreElements()) {
			LocalTypeItem lti = (LocalTypeItem)(e.nextElement());
			entryNode = OAPIModify.insertElementIntoEntry(entryNode,
												(lti.getDefault()).getType().toString(), 
												(lti).getDefault().getPath(), 
												(lti).getDefault().getValue());
		} 
		return entryNode;
	}
	
/**
* <p> This returns the Required object. </p>
*
* @return Required object
*/
	public Required getRequired() {
		return this.required;
	}
	
/**
* <p> This returns the OLIF template document object. </p>
*
* @return OLIF template document object
*/
	public Document getOlifTemplateDocument() {
		return (Document)((Node)this.olifTemplateDocument).cloneNode(true);
	}
	
/**
* <p> This returns the Entry template node object. </p>
*
* @return Entry template node object
*/
	public Node getEntryTemplateNode() throws OAPIException {
		return this.entryTemplateNode.cloneNode(true);
	}

/**
* <p> This returns the OLIF template object. </p>
*
* @return OLIF template object
* @throws OAPISerializeException
*/
	public Olif getOlifTemplate() throws OAPISerializeException {
		return OAPISerialize.convertToOlif(this.olifTemplateDocument);
	}
	
/**
* <p> This returns the Entry template object. </p>
*
* @param userID String to identify Entry locally
* @return Entry template object
* @throws OAPIException
*/
	public Entry getEntryTemplate(String userID) throws OAPIException {
		Entry entry = OAPISerialize.convertToEntry((Document)this.entryTemplateNode);
		entry.setConceptUniversalId(getUUID());
		entry.setConceptUserId(userID);
		return entry;
	}
	
/**
* <p> This creates a UUID for each Entry node.</p>
*/
	private static String getUUID() {
		UUID uuid = new UUID();
		return uuid.toString();
	}

}
